#!/usr/bin/env python
"""
This program takes information about a release in the ReleaseConfig file
and inserts the information in various files.  It helps to keep the files
in sync because it never forgets which files need to be updated, nor what
information should be inserted in each file.

It takes one parameter that says what the mode of update should be, which
may be one of 'develop', 'release', or 'post'.

In 'develop' mode, which is what someone would use as part of their own
development practices, the release type is forced to be 'alpha' and the
patch level is the string 'yyyymmdd'.  Otherwise, it's the same as the
'release' mode.

In 'release' mode, the release type is taken from ReleaseConfig and the
patch level is calculated from the release date for non-final runs and
taken from the version tuple for final runs.  It then inserts information
in various files:
  - The RELEASE header line in src/CHANGES.txt and src/Announce.txt.
  - The version string at the top of src/RELEASE.txt.
  - The version string in the 'default_version' variable in SConstruct
    and testing/framework/TestSCons.py.
  - The copyright years in SConstruct and testing/framework/TestSCons.py.
  - The month and year (used for documentation) in SConstruct.
  - The unsupported and deprecated Python floors in testing/framework/TestSCons.py
    and src/engine/SCons/Script/Main.py
  - The version string in the filenames in README.

In 'post' mode, files are prepared for the next release cycle:
  - In ReleaseConfig, the version tuple is updated for the next release
    by incrementing the release number (either minor or micro, depending
    on the branch) and resetting release type to 'alpha'.
  - A blank template replaces src/RELEASE.txt.
  - A new section to accumulate changes is added to src/CHANGES.txt and
    src/Announce.txt.
"""
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
from __future__ import print_function

__revision__ = "bin/update-release-info.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import os
import sys
import time
import re

DEBUG = os.environ.get('DEBUG', 0)

# Evaluate parameter

if len(sys.argv) < 2:
    mode = 'develop'
else:
    mode = sys.argv[1]
    if mode not in ['develop', 'release', 'post']:
        print(("""ERROR: `%s' as a parameter is invalid; it must be one of
\tdevelop, release, or post.  The default is develop.""" % mode))
        sys.exit(1)

# Get configuration information

config = dict()
with open('ReleaseConfig') as f:
    releaseconfig = f.read()
exec(releaseconfig, globals(), config)


try:
    version_tuple = config['version_tuple']
    unsupported_version = config['unsupported_python_version']
    deprecated_version = config['deprecated_python_version']
except KeyError:
    print('''ERROR: Config file must contain at least version_tuple,
\tunsupported_python_version, and deprecated_python_version.''')
    sys.exit(1)
if DEBUG: print('version tuple', version_tuple)
if DEBUG: print('unsupported Python version', unsupported_version)
if DEBUG: print('deprecated Python version', deprecated_version)

try:
    release_date = config['release_date']
except KeyError:
    release_date = time.localtime()[:6]
else:
    if len(release_date) == 3:
        release_date = release_date + time.localtime()[3:6]
    if len(release_date) != 6:
        print('''ERROR: Invalid release date''', release_date)
        sys.exit(1)
if DEBUG: print('release date', release_date)

if mode == 'develop' and version_tuple[3] != 'alpha':
    version_tuple ==  version_tuple[:3] + ('alpha', 0)
if len(version_tuple) > 3 and version_tuple[3] != 'final':
    if mode == 'develop':
        version_tuple = version_tuple[:4] + ('yyyymmdd',)
    else:
        yyyy,mm,dd,_,_,_ = release_date
        version_tuple = version_tuple[:4] + ((yyyy*100 + mm)*100 + dd,)
version_string = '.'.join(map(str, version_tuple))
if len(version_tuple) > 3:
    version_type = version_tuple[3]
else:
    version_type = 'final'
if DEBUG: print('version string', version_string)

if version_type not in ['alpha', 'beta', 'candidate', 'final']:
    print(("""ERROR: `%s' is not a valid release type in version tuple;
\tit must be one of alpha, beta, candidate, or final""" % version_type))
    sys.exit(1)

try:
    month_year = config['month_year']
except KeyError:
    if version_type == 'alpha':
        month_year = 'MONTH YEAR'
    else:
        month_year =  time.strftime('%B %Y', release_date + (0,0,0))
if DEBUG: print('month year', month_year)

try:
    copyright_years = config['copyright_years']
except KeyError:
    copyright_years = '2001 - %d'%(release_date[0] + 1)
if DEBUG: print('copyright years', copyright_years)

class UpdateFile(object):
    """
    XXX
    """

    def __init__(self, file, orig = None):
        '''
        '''
        if orig is None: orig = file
        try:
            with open(orig, 'r') as f:
                self.content = f.read()
        except IOError:
            # Couldn't open file; don't try to write anything in __del__
            self.file = None
            raise
        else:
            self.file = file
            if file == orig:
                # so we can see if it changed
                self.orig = self.content
            else:
                # pretend file changed
                self.orig = ''

    def sub(self, pattern, replacement, count = 1):
        '''
        XXX
        '''
        self.content = re.sub(pattern, replacement, self.content, count)

    def replace_assign(self, name, replacement, count = 1):
        '''
        XXX
        '''
        self.sub('\n' + name + ' = .*', '\n' + name + ' = ' + replacement)

    # Determine the pattern to match a version

    _rel_types = r'(alpha|beta|candidate|final)'
    match_pat = r'\d+\.\d+\.\d+\.' + _rel_types + r'\.(\d+|yyyymmdd)'
    match_rel = re.compile(match_pat)

    def replace_version(self, replacement = version_string, count = 1):
        '''
        XXX
        '''
        self.content = self.match_rel.sub(replacement, self.content, count)

    # Determine the release date and the pattern to match a date
    # Mon, 05 Jun 2010 21:17:15 -0700
    # NEW DATE WILL BE INSERTED HERE

    if mode == 'develop':
        new_date = 'NEW DATE WILL BE INSERTED HERE'
    else:
        min = (time.daylight and time.altzone or time.timezone)//60
        hr = min // 60
        min = -(min % 60 + hr * 100)
        new_date =  (time.strftime('%a, %d %b %Y %X', release_date + (0,0,0))
                         + ' %+.4d' % min)

    _days = r'(Sun|Mon|Tue|Wed|Thu|Fri|Sat)'
    _months = r'(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oce|Nov|Dec)'
    match_date = r''.join([_days, r', \d\d ', _months, r' \d\d\d\d \d\d:\d\d:\d\d [+-]\d\d\d\d'])
    match_date = re.compile(match_date)

    def replace_date(self, replacement = new_date, count = 1):
        '''
        XXX
        '''
        self.content = self.match_date.sub(replacement, self.content, count)

    def __del__(self):
        '''
        XXX
        '''
        if self.file is not None and self.content != self.orig:
            print('Updating ' + self.file + '...')
            with open(self.file, 'w') as f:
                f.write(self.content)

if mode == 'post':
    # Set up for the next release series.

    if version_tuple[2]:
        # micro release, increment micro value
        minor = version_tuple[1]
        micro = version_tuple[2] + 1
    else:
        # minor release, increment minor value
        minor = version_tuple[1] + 1
        micro = 0
    new_tuple = (version_tuple[0], minor, micro, 'alpha', 0)
    new_version = '.'.join(map(str, new_tuple[:4])) + '.yyyymmdd'

    # Update ReleaseConfig

    t = UpdateFile('ReleaseConfig')
    if DEBUG: t.file = '/tmp/ReleaseConfig'
    t.replace_assign('version_tuple', str(new_tuple))

    # Update src/CHANGES.txt

    t = UpdateFile(os.path.join('src', 'CHANGES.txt'))
    if DEBUG: t.file = '/tmp/CHANGES.txt'
    t.sub('(\nRELEASE .*)', r"""\nRELEASE  VERSION/DATE TO BE FILLED IN LATER\n
  From John Doe:\n
    - Whatever John Doe did.\n
\1""")

    # Update src/RELEASE.txt

    t = UpdateFile(os.path.join('src', 'RELEASE.txt'),
                   os.path.join('template', 'RELEASE.txt'))
    if DEBUG: t.file = '/tmp/RELEASE.txt'
    t.replace_version(new_version)

    # Update src/Announce.txt

    t = UpdateFile(os.path.join('src', 'Announce.txt'))
    if DEBUG: t.file = '/tmp/Announce.txt'
    t.sub('\nRELEASE .*', '\nRELEASE VERSION/DATE TO BE FILLED IN LATER')
    announce_pattern = """(
  Please note the following important changes scheduled for the next
  release:
)"""
    announce_replace = (r"""\1
    --  FEATURE THAT WILL CHANGE\n
  Please note the following important changes since release """
            + '.'.join(map(str, version_tuple[:3])) + ':\n')
    t.sub(announce_pattern, announce_replace)

    # Write out the last update and exit

    t = None
    sys.exit()

# Update src/CHANGES.txt

t = UpdateFile(os.path.join('src', 'CHANGES.txt'))
if DEBUG: t.file = '/tmp/CHANGES.txt'
t.sub('\nRELEASE .*', '\nRELEASE ' + version_string + ' - ' + t.new_date)

# Update src/RELEASE.txt

t = UpdateFile(os.path.join('src', 'RELEASE.txt'))
if DEBUG: t.file = '/tmp/RELEASE.txt'
t.replace_version()

# Update src/Announce.txt

t = UpdateFile(os.path.join('src', 'Announce.txt'))
if DEBUG: t.file = '/tmp/Announce.txt'
t.sub('\nRELEASE .*', '\nRELEASE ' + version_string + ' - ' + t.new_date)


# Update SConstruct

t = UpdateFile('SConstruct')
if DEBUG: t.file = '/tmp/SConstruct'
t.replace_assign('month_year', repr(month_year))
t.replace_assign('copyright_years', repr(copyright_years))
t.replace_assign('default_version', repr(version_string))

# Update README

t = UpdateFile('README.rst')
if DEBUG: t.file = '/tmp/README.rst'
t.sub('-' + t.match_pat + r'\.', '-' + version_string + '.', count = 0)
for suf in ['tar', 'win32', 'zip', 'rpm', 'exe', 'deb']:
    t.sub(r'-(\d+\.\d+\.\d+)\.%s' % suf,
          '-%s.%s' % (version_string, suf),
          count = 0)

# Update testing/framework/TestSCons.py

t = UpdateFile(os.path.join('testing','framework', 'TestSCons.py'))
if DEBUG: t.file = '/tmp/TestSCons.py'
t.replace_assign('copyright_years', repr(copyright_years))
t.replace_assign('default_version', repr(version_string))
#??? t.replace_assign('SConsVersion', repr(version_string))
t.replace_assign('python_version_unsupported', str(unsupported_version))
t.replace_assign('python_version_deprecated', str(deprecated_version))

# Update Script/Main.py

t = UpdateFile(os.path.join('src', 'engine', 'SCons', 'Script', 'Main.py'))
if DEBUG: t.file = '/tmp/Main.py'
t.replace_assign('unsupported_python_version', str(unsupported_version))
t.replace_assign('deprecated_python_version', str(deprecated_version))

# Update doc/user/main.{in,xml}

docyears = '2004 - %d' % release_date[0]
if os.path.exists(os.path.join('doc', 'user', 'main.in')):
    # this is no longer used as of Dec 2013
    t = UpdateFile(os.path.join('doc', 'user', 'main.in'))
    if DEBUG: t.file = '/tmp/main.in'
    ## TODO debug these
    #t.sub('<pubdate>[^<]*</pubdate>', '<pubdate>' + docyears + '</pubdate>')
    #t.sub('<year>[^<]*</year>', '<year>' + docyears + '</year>')

t = UpdateFile(os.path.join('doc', 'user', 'main.xml'))
if DEBUG: t.file = '/tmp/main.xml'
t.sub('<pubdate>[^<]*</pubdate>', '<pubdate>' + docyears + '</pubdate>')
t.sub('<year>[^<]*</year>', '<year>' + docyears + '</year>')

# Write out the last update

t = None

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
# -*- Python -*-
from __future__ import division, print_function

import sys
from math import log, ceil
from optparse import OptionParser
import subprocess

# crack the command line
parser = OptionParser(
             usage="%prog lower-revision upper-revision test_script [arg1 ...]",
             description="Binary search for a bug in a SVN checkout")
(options,script_args) = parser.parse_args()

# make sure we have sufficient parameters
if len(script_args) < 1:
    parser.error("Need a lower revision")
elif len(script_args) < 2:
    parser.error("Need an upper revision")
elif len(script_args) < 3:
    parser.error("Need a script to run")

# extract our starting values
lower = int(script_args[0])
upper = int(script_args[1])
script = script_args[2:]

# print an error message and quit
def error(s):
    print("******", s, "******", file=sys.stderr)
    sys.exit(1)    

# update to the specified version and run test
def testfail(revision):
    "Return true if test fails"
    print("Updating to revision", revision)
    if subprocess.call(["svn","up","-qr",str(revision)]) != 0:
        m = "SVN did not update properly to revision %d"
        raise RuntimeError(m % revision)
    return subprocess.call(script,shell=False) != 0

# confirm that the endpoints are different
print("****** Checking upper bracket", upper)
upperfails = testfail(upper)
print("****** Checking lower bracket", lower)
lowerfails = testfail(lower)
if upperfails == lowerfails:
    error("Upper and lower revisions must bracket the failure")

# binary search for transition
msg = "****** max %d revisions to test (bug bracketed by [%d,%d])"
while upper-lower > 1:
    print(msg % (ceil(log(upper-lower,2)), lower, upper))

    mid = (lower + upper)//2
    midfails = testfail(mid)
    if midfails == lowerfails:
        lower = mid
        lowerfails = midfails
    else:
        upper = mid
        upperfails = midfails

# show which revision was first to fail
if upperfails != lowerfails: lower = upper
print("The error was caused by revision", lower)

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/scons-time/func/function.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify use of the --func and --function options to select functions
other than the default _main().
"""

import TestSCons_time

test = TestSCons_time.TestSCons_time(match = TestSCons_time.match_re)

try:
    import pstats
except ImportError:
    test.skip_test('No pstats module, skipping test.\n')

test.profile_data('foo.prof', 'prof.py', '_main', """\
def f1():
    pass
def f3():
    pass
def _main():
    f1()
    f3()
""")

expect1 = r'\d.\d\d\d prof\.py:1\(f1\)' + '\n'
expect3 = r'\d.\d\d\d prof\.py:3\(f3\)' + '\n'

test.run(arguments = 'func --func f1 foo.prof', stdout = expect1)

test.run(arguments = 'func --function f3 foo.prof', stdout = expect3)

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/Removed/SourceSignatures/Old/basic.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import os
import re

import TestSCons

test = TestSCons.TestSCons(match = TestSCons.match_re_dotall)


base_sconstruct_contents = """\
SetOption('warn', 'deprecated-source-signatures')
def build(env, target, source):
    with open(str(target[0]), 'wt') as ofp, open(str(source[0]), 'rt') as ifp:
        ofp.write(ifp.read())
B = Builder(action = build)
env = Environment(BUILDERS = { 'B' : B })
env.B(target = 'f1.out', source = 'f1.in')
env.B(target = 'f2.out', source = 'f2.in')
env.B(target = 'f3.out', source = 'f3.in')
env.B(target = 'f4.out', source = 'f4.in')
"""

def write_SConstruct(test, sigtype):
    contents = base_sconstruct_contents
    if sigtype:
         contents = contents + ("\nSourceSignatures('%s')\n" % sigtype)
    test.write('SConstruct', contents)


expect = TestSCons.re_escape("""
scons: warning: The env.SourceSignatures() method is deprecated;
\tconvert your build to use the env.Decider() method instead.
""") + TestSCons.file_expr

write_SConstruct(test, 'timestamp')

test.write('f1.in', "f1.in\n")
test.write('f2.in', "f2.in\n")
test.write('f3.in', "f3.in\n")
test.write('f4.in', "f4.in\n")

test.run(arguments = 'f1.out f3.out', stderr = expect)

test.run(arguments = 'f1.out f2.out f3.out f4.out',
         stdout = re.escape(test.wrap_stdout("""\
scons: `f1.out' is up to date.
build(["f2.out"], ["f2.in"])
scons: `f3.out' is up to date.
build(["f4.out"], ["f4.in"])
""")),
         stderr = expect)


os.utime(test.workpath('f1.in'), 
         (os.path.getatime(test.workpath('f1.in')),
          os.path.getmtime(test.workpath('f1.in'))+10))
os.utime(test.workpath('f3.in'), 
         (os.path.getatime(test.workpath('f3.in')),
          os.path.getmtime(test.workpath('f3.in'))+10))

test.run(arguments = 'f1.out f2.out f3.out f4.out',
         stdout = re.escape(test.wrap_stdout("""\
build(["f1.out"], ["f1.in"])
scons: `f2.out' is up to date.
build(["f3.out"], ["f3.in"])
scons: `f4.out' is up to date.
""")),
         stderr = expect)


# Switching to content signatures from timestamps should rebuild,
# because we didn't record the content signatures last time.

write_SConstruct(test, 'MD5')

test.not_up_to_date(arguments = 'f1.out f2.out f3.out f4.out', stderr = expect)


test.sleep()

test.write('f1.in', "f1.in\n")
test.write('f2.in', "f2.in\n")
test.write('f3.in', "f3.in\n")
test.write('f4.in', "f4.in\n")

test.up_to_date(arguments = 'f1.out f2.out f3.out f4.out', stderr = None)


test.touch('f1.in', os.path.getmtime(test.workpath('f1.in'))+10)
test.touch('f3.in', os.path.getmtime(test.workpath('f3.in'))+10)

test.up_to_date(arguments = 'f1.out f2.out f3.out f4.out', stderr = None)


write_SConstruct(test, None)

test.up_to_date(arguments = 'f1.out f2.out f3.out f4.out', stderr = None)


test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "src/engine/SCons/Node/PythonTests.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import sys
import unittest

import SCons.Errors
import SCons.Node.Python

class ValueTestCase(unittest.TestCase):

    def test_Value(self):
        """Test creating a Value() object
        """
        v1 = SCons.Node.Python.Value('a')
        assert v1.value == 'a', v1.value

        value2 = 'a'
        v2 = SCons.Node.Python.Value(value2)
        assert v2.value == value2, v2.value
        assert v2.value is value2, v2.value

        assert not v1 is v2
        assert v1.value == v2.value

        v3 = SCons.Node.Python.Value('c', 'cb')
        assert v3.built_value == 'cb'

    def test_build(self):
        """Test "building" a Value Node
        """
        class fake_executor(object):
            def __call__(self, node):
                node.write('faked')

        v1 = SCons.Node.Python.Value('b', 'built')
        v1.executor = fake_executor()
        v1.build()
        assert v1.built_value == 'built', v1.built_value

        v2 = SCons.Node.Python.Value('b')
        v2.executor = fake_executor()
        v2.build()
        assert v2.built_value == 'faked', v2.built_value

    def test_read(self):
        """Test the Value.read() method
        """
        v1 = SCons.Node.Python.Value('a')
        x = v1.read()
        assert x == 'a', x

    def test_write(self):
        """Test the Value.write() method
        """
        v1 = SCons.Node.Python.Value('a')
        assert v1.value == 'a', v1.value
        assert not hasattr(v1, 'built_value')

        v1.write('new')
        assert v1.value == 'a', v1.value
        assert v1.built_value == 'new', v1.built_value

    def test_get_csig(self):
        """Test calculating the content signature of a Value() object
        """
        v1 = SCons.Node.Python.Value('aaa')
        csig = v1.get_csig(None)
        assert csig == 'aaa', csig

        v2 = SCons.Node.Python.Value(7)
        csig = v2.get_csig(None)
        assert csig == '7', csig

        v3 = SCons.Node.Python.Value(None)
        csig = v3.get_csig(None)
        assert csig == 'None', csig

class ValueNodeInfoTestCase(unittest.TestCase):
    def test___init__(self):
        """Test ValueNodeInfo initialization"""
        vvv = SCons.Node.Python.Value('vvv')
        ni = SCons.Node.Python.ValueNodeInfo()

class ValueBuildInfoTestCase(unittest.TestCase):
    def test___init__(self):
        """Test ValueBuildInfo initialization"""
        vvv = SCons.Node.Python.Value('vvv')
        bi = SCons.Node.Python.ValueBuildInfo()

if __name__ == "__main__":
    unittest.main()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
from __future__ import print_function

"""
Verify that a failed build action with -j works as expected.
"""

__revision__ = "test/GetBuildFailures/parallel.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import TestSCons

_python_ = TestSCons._python_

try:
    import threading
except ImportError:
    # if threads are not supported, then
    # there is nothing to test
    TestCmd.no_result()
    sys.exit()


test = TestSCons.TestSCons()

# We want to verify that -j 4 starts all four jobs, the first and last of
# which fail and the second and third of which succeed, and then stops
# processing due to the build failures.  To try to control the timing,
# the created build scripts use marker directories to avoid doing their
# processing until the previous script has finished.

contents = r"""\
import os.path
import sys
import time
wait_marker = sys.argv[1] + '.marker'
write_marker = sys.argv[2] + '.marker'
if wait_marker != '-.marker':
    while not os.path.exists(wait_marker):
        time.sleep(1)
if 'mypass.py' in sys.argv[0]:
    with open(sys.argv[3], 'wb') as ofp, open(sys.argv[4], 'rb') as ifp:
        ofp.write(ifp.read())
    exit_value = 0
elif 'myfail.py' in sys.argv[0]:
    exit_value = 1
if write_marker != '-.marker':
    os.mkdir(write_marker)
sys.exit(exit_value)
"""

test.write('mypass.py', contents)
test.write('myfail.py', contents)

test.write('SConstruct', """\
DefaultEnvironment(tools=[])
Command('f3', 'f3.in', r'@%(_python_)s mypass.py -  f3 $TARGET $SOURCE')
Command('f4', 'f4.in', r'@%(_python_)s myfail.py f3 f4 $TARGET $SOURCE')
Command('f5', 'f5.in', r'@%(_python_)s myfail.py f4 f5 $TARGET $SOURCE')
Command('f6', 'f6.in', r'@%(_python_)s mypass.py f5 -  $TARGET $SOURCE')

def print_build_failures():
    from SCons.Script import GetBuildFailures
    for bf in sorted(GetBuildFailures(), key=lambda t: t.filename or 'None'):
        print("%%s failed:  %%s" %% (bf.node, bf.errstr))

import atexit
atexit.register(print_build_failures)
""" % locals())

test.write('f3.in', "f3.in\n")
test.write('f4.in', "f4.in\n")
test.write('f5.in', "f5.in\n")
test.write('f6.in', "f6.in\n")

test.run(arguments = '-Q -j 4 .',
         status = 2,
         stderr = None)

f4_error = "scons: *** [f4] Error 1\n"
f5_error = "scons: *** [f5] Error 1\n"

error_45 = f4_error + f5_error
error_54 = f5_error + f4_error

if test.stderr() not in [error_45, error_54]:
    print("Did not find the following output in list of expected strings:")
    print(test.stderr(), end=' ')
    test.fail_test()

# We jump through hoops above to try to make sure that the individual
# commands execute and exit in the order we want, but we still can't be
# 100% sure that SCons will actually detect and record the failures in
# that order; the thread for f5 may detect its command's failure before
# the thread for f4.  Just sidestep the issue by allowing the failure
# strings in the output to come in either order.  If there's a genuine
# problem in the way things get ordered, it'll show up in stderr.

f4_failed = "f4 failed:  Error 1\n"
f5_failed = "f5 failed:  Error 1\n"

failed_45 = f4_failed + f5_failed
failed_54 = f5_failed + f4_failed

if test.stdout() not in [failed_45, failed_54]:
    print("Did not find the following output in list of expected strings:")
    print(test.stdout(), end=' ')
    test.fail_test()

test.must_match(test.workpath('f3'), 'f3.in\n')
test.must_not_exist(test.workpath('f4'))
test.must_not_exist(test.workpath('f5'))
test.must_match(test.workpath('f6'), 'f6.in\n')



test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/Java/JARCHDIR.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Test that when JARCHDIR that our command to create .jar files
correctly finds all the .class files (by putting -C in front
of each class file argument).

Includes logic to make sure that expansions of $JARCHDIR that include
${TARGET} or ${SOURCE} work.
"""

import os

import TestSCons

test = TestSCons.TestSCons()
# Keep this logic because it skips the test if javac or jar not found.
where_javac, java_version = test.java_where_javac()
where_jar = test.java_where_jar()

test.write('SConstruct', """
DefaultEnvironment(tools=[])
dir = 'dist'
env = Environment(tools    = ['javac', 'jar'],
                  JARCHDIR = dir)
bin = env.Java(dir, Dir('./'))
jar = env.Jar(File('c.jar', dir), bin)

# Make sure we handle class files with $ in them, such as typically
# created for inner classes.
env = env.Clone(JARCHDIR = '.')
inner = env.Jar('inner.jar', 'Inner$$Class.class')

# Commented out as this logic doesn't work as is.
# target_env = env.Clone(JARCHDIR = '${TARGET.dir}')
# target_env.Jar('out/t.jar', 'in/t.class')

source_env = env.Clone(JARCHDIR = '${SOURCE.dir}')
source_env.Jar('out/s.jar', 'in/s.class')

Default(bin, jar, inner)
""" % locals())

test.subdir('in')

test.write('a.java', """\
package foo.bar;
public class a {}
""")

test.write('b.java', """\
package foo.bar;
public class b {}
""")

test.write(['in', 's.class'], "s.class\n")

# Okay, this is bogus, but we're going with it for testing purposes.
# If jar gets a command line like:
#
#	jar cf out/t.jar -C out /tmp/tmpXYZZY/in/t.class
#
# Empirically, it doesn't seem to treat the absolute path name
# of the argument class file as an absolute path, but looks for
# "out/tmp/tmpXYZZY/in/t.class".  SCons, however, still looks for it in
# the path name specified on the command line.  To make this test work,
# we're going to just create the t.class file in both locations, and
# we can revisit this if someone actually tries to use ${TARGET.dir}
# in a real-life expansion.  Right now, it at least makes sure things
# don't blow up (i.e., validates that we pass the right arguments to
# env.subst() in the code that handle jar).

# p = test.workpath('out')
# for d in test.workpath('in').split(os.sep):
#     p = p + d
#     test.subdir(p)
#     p = p + os.sep

# test.write([p, 't.class'], "t.class\n")
test.write(['in', 't.class'], "t.class\n")

test.write('Inner$Class.class', "Inner$Class.class\n")

test.run(arguments = '.')

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



import sys

if sys.platform == 'win32':
    args = sys.argv[1:]
    inf = None
    while args:
        a = args[0]
        if a == '-o':
            out = args[1]
            args = args[2:]
            continue
        args = args[1:]
        if not a[0] in "/-":
            if not inf:
                inf = a
            continue
        if a[:3] == '/Fo': out = a[3:]
    with open(inf, 'rb') as ifp, open(out, 'wb') as ofp:
        for l in ifp.readlines():
            if l[:3] != b'#as':
                ofp.write(l)
    sys.exit(0)

else:
    import getopt
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'co:')
    except getopt.GetoptError:
        # we may be called with --version, just quit if so
        sys.exit(0)
    for opt, arg in opts:
        if opt == '-o': out = arg
    if args:
        with open(args[0], 'rb') as ifp, open(out, 'wb') as ofp:
            for l in ifp.readlines():
                if l[:3] != b'#as':
                    ofp.write(l)
    sys.exit(0)



import os
import sys
import subprocess

if __name__ == '__main__':
    path = os.path.join(os.path.dirname(os.path.relpath(__file__)), 'wrapper.out')
    if '--version' not in sys.argv and '-dumpversion' not in sys.argv:
        with open(path, 'wb') as f:
            f.write(b"wrapper.py\n")
    subprocess.call(sys.argv[1:])



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/scons-time/run/option/verbose.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify that the run -v and --verbose options display command output.
"""

import sys
import re
import os
import TestSCons_time

_python_ = re.escape('"' + sys.executable + '"')


test = TestSCons_time.TestSCons_time(match = TestSCons_time.match_re,
                                     diff = TestSCons_time.diff_re)

scons_py = re.escape(test.workpath('src', 'script', 'scons.py'))
src_engine = re.escape(test.workpath('src', 'engine'))

tmp_scons_time = test.tempdir_re()
tmp_scons_time_foo = test.tempdir_re('foo')


test.write_fake_scons_py()

foo_tar_gz = test.write_sample_project('foo.tar.gz')

expect = """\
%(scons_py)s
    --version
SCONS_LIB_DIR = %(src_engine)s
SConstruct file directory: %(tmp_scons_time_foo)s
""" % locals()

test.run(arguments = 'run -q foo.tar.gz', stdout = expect)

test.must_exist('foo-000-0.log',
                'foo-000-0.prof',
                'foo-000-1.log',
                'foo-000-1.prof',
                'foo-000-2.log',
                'foo-000-2.prof')

time_re = r'\[\d\d:\d\d:\d\d\]'

scons_flags = '--debug=count --debug=memory --debug=time --debug=memoizer'


expect = """\
scons-time%(time_re)s: cd %(tmp_scons_time)s
scons-time%(time_re)s: tar xzf %(foo_tar_gz)s
scons-time%(time_re)s: cd foo
scons-time%(time_re)s: find \\* -type f | xargs cat > /dev/null
scons-time%(time_re)s: export SCONS_LIB_DIR=%(src_engine)s
scons-time%(time_re)s: %(_python_)s %(scons_py)s --version
%(scons_py)s
    --version
SCONS_LIB_DIR = %(src_engine)s
SConstruct file directory: %(tmp_scons_time_foo)s
scons-time%(time_re)s: %(_python_)s %(scons_py)s %(scons_flags)s --profile=%(prof0)s --help 2>&1 \\| tee %(log0)s
%(scons_py)s
    --debug=count
    --debug=memory
    --debug=time
    --debug=memoizer
    --profile=%(prof0)s
    --help
SCONS_LIB_DIR = %(src_engine)s
SConstruct file directory: %(tmp_scons_time_foo)s
scons-time%(time_re)s: %(_python_)s %(scons_py)s %(scons_flags)s --profile=%(prof1)s  2>&1 \\| tee %(log1)s
%(scons_py)s
    --debug=count
    --debug=memory
    --debug=time
    --debug=memoizer
    --profile=%(prof1)s
SCONS_LIB_DIR = %(src_engine)s
SConstruct file directory: %(tmp_scons_time_foo)s
scons-time%(time_re)s: %(_python_)s %(scons_py)s %(scons_flags)s --profile=%(prof2)s  2>&1 \\| tee %(log2)s
%(scons_py)s
    --debug=count
    --debug=memory
    --debug=time
    --debug=memoizer
    --profile=%(prof2)s
SCONS_LIB_DIR = %(src_engine)s
SConstruct file directory: %(tmp_scons_time_foo)s
scons-time%(time_re)s: cd .*
"""
if 'PRESERVE' not in os.environ or not os.environ['PRESERVE']:
    expect += """scons-time%(time_re)s: rm -rf %(tmp_scons_time)s
"""

foo_tar_gz = re.escape(foo_tar_gz)

log0 = re.escape(test.workpath('foo-001-0.log'))
log1 = re.escape(test.workpath('foo-001-1.log'))
log2 = re.escape(test.workpath('foo-001-2.log'))

prof0 = re.escape(test.workpath('foo-001-0.prof'))
prof1 = re.escape(test.workpath('foo-001-1.prof'))
prof2 = re.escape(test.workpath('foo-001-2.prof'))

test.run(arguments = 'run -v foo.tar.gz', stdout = expect % locals())

test.must_exist('foo-001-0.log',
                'foo-001-0.prof',
                'foo-001-1.log',
                'foo-001-1.prof',
                'foo-001-2.log',
                'foo-001-2.prof')

log0 = re.escape(test.workpath('foo-002-0.log'))
log1 = re.escape(test.workpath('foo-002-1.log'))
log2 = re.escape(test.workpath('foo-002-2.log'))

prof0 = re.escape(test.workpath('foo-002-0.prof'))
prof1 = re.escape(test.workpath('foo-002-1.prof'))
prof2 = re.escape(test.workpath('foo-002-2.prof'))

test.run(arguments = 'run --verbose foo.tar.gz', stdout = expect % locals())


test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/MSVC/query_vcbat.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import TestSCons

test = TestSCons.TestSCons(match = TestSCons.match_re)

test.skip_if_not_msvc()

#####
# Test the basics

test.write('SConstruct', """
#from SCons.Tool.MSCommon.misc import FindMSVSBatFile, \\
#                                       ParseBatFile, \\
#                                       MergeMSVSBatFile
from SCons.Tool.MSCommon import query_versions
#env = Environment(tools = ['mingw'])
DefaultEnvironment(tools = [])
#for v in [9, 8, 7.1, 7]:
#    print " ==== Testing for version %s ==== " % str(v)
#    bat = FindMSVSBatFile(v)
#    print bat
#    if bat:
#        d = ParseBatFile(bat)
#        for k, v in d.items():
#            print k, v
#MergeMSVSBatFile(env, 9.0)
#print(env['ENV']['PATH'])
print(query_versions())
""")

test.run(stderr = None)
test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/CPPFLAGS.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import sys

import TestSCons

_python_ = TestSCons._python_

test = TestSCons.TestSCons()

# Writing this to accomodate both our in-line tool chain and the
# MSVC command lines is too hard, and will be completely unnecessary
# some day when we separate our tests.  Punt for now.
if sys.platform == 'win32':
    test.skip_test('Skipping on win32.\n')



if sys.platform == 'win32':

    test.write('mylink.py', r"""
import sys
args = sys.argv[1:]
while args:
    a = args[0]
    if a[0] != '/':
        break
    args.pop(0)
    if a[:5] == '/OUT:': out = a[5:]
with open(out, 'w') as ofp, open(args[0], 'r') as ifp:
    for l in ifp.readlines():
        if l[:5] != '#link':
            ofp.write(l)
sys.exit(0)
""")

else:

    test.write('mylink.py', r"""
import getopt
import sys
opts, args = getopt.getopt(sys.argv[1:], 'o:s:')
for opt, arg in opts:
    if opt == '-o': out = arg
with open(out, 'w') as ofp:
    for f in args:
        with open(f, 'r') as ifp:
            for l in ifp.readlines():
                if l[:5] != '#link':
                    ofp.write(l)
sys.exit(0)
""")

test.write('mygcc.py', r"""
import getopt
import os
import sys
compiler = sys.argv[1]
clen = len(compiler) + 1
opts, args = getopt.getopt(sys.argv[2:], 'co:xf:K:')
for opt, arg in opts:
    if opt == '-o': out = arg
    elif opt == '-x':
        with open('mygcc.out', 'a') as f:
            f.write(compiler + "\n")
with open(out, 'w') as ofp, open(args[0], 'r') as ifp:
    for l in ifp.readlines():
        if l[:clen] != '#' + compiler:
            ofp.write(l)
sys.exit(0)
""")



test.write('SConstruct', """
env = Environment(CPPFLAGS = '-x',
                  LINK = r'%(_python_)s mylink.py',
                  LINKFLAGS = [],
                  CC = r'%(_python_)s mygcc.py cc',
                  CXX = r'%(_python_)s mygcc.py c++',
                  CXXFLAGS = [],
                  FORTRAN = r'%(_python_)s mygcc.py g77',
                  OBJSUFFIX = '.obj',
                  PROGSUFFIX = '.exe')
env.Program(target = 'foo', source = Split('test1.c test2.cpp test3.F'))
""" % locals())

test.write('test1.c', r"""test1.c
#cc
#link
""")

test.write('test2.cpp', r"""test2.cpp
#c++
#link
""")

test.write('test3.F', r"""test3.F
#g77
#link
""")

test.run(arguments = '.', stderr=None)

test.must_match('test1.obj', "test1.c\n#link\n")
test.must_match('test2.obj', "test2.cpp\n#link\n")
test.must_match('test3.obj', "test3.F\n#link\n")
test.must_match('foo.exe',   "test1.c\ntest2.cpp\ntest3.F\n")
if TestSCons.case_sensitive_suffixes('.F', '.f'):
    test.must_match('mygcc.out', "cc\nc++\ng77\n")
else:
    test.must_match('mygcc.out', "cc\nc++\n")   

test.write('SConstruct', """
env = Environment(CPPFLAGS = '-x',
                  SHLINK = r'%(_python_)s mylink.py',
                  SHLINKFLAGS = [],
                  CC = r'%(_python_)s mygcc.py cc',
                  CXX = r'%(_python_)s mygcc.py c++',
                  CXXFLAGS = [],
                  FORTRAN = r'%(_python_)s mygcc.py g77',
                  OBJSUFFIX = '.obj',
                  SHOBJPREFIX = '',
                  SHOBJSUFFIX = '.shobj',
                  PROGSUFFIX = '.exe')
env.SharedLibrary(target = File('foo.bar'),
                  source = Split('test1.c test2.cpp test3.F'))
""" % locals())

test.write('test1.c', r"""test1.c
#cc
#link
""")

test.write('test2.cpp', r"""test2.cpp
#c++
#link
""")

test.write('test3.F', r"""test3.F
#g77
#link
""")

test.unlink('mygcc.out')
test.unlink('test1.obj')
test.unlink('test2.obj')
test.unlink('test3.obj')

test.run(arguments = '.', stderr = None)

test.must_match('test1.shobj', "test1.c\n#link\n")
test.must_match('test2.shobj', "test2.cpp\n#link\n")
test.must_match('test3.shobj', "test3.F\n#link\n")
test.must_match('foo.bar',        "test1.c\ntest2.cpp\ntest3.F\n")
if TestSCons.case_sensitive_suffixes('.F', '.f'):
    test.must_match('mygcc.out', "cc\nc++\ng77\n")
else:
    test.must_match('mygcc.out', "cc\nc++\n")   

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/Builder/multi/different-order.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify that a "multi" builder can NOT be called multiple times with
target lists that have different orders.  This is intentional; the
order of the targets matter to the builder because the build command
can contain things like ${TARGET[0]}.
"""

import TestSCons

test = TestSCons.TestSCons(match=TestSCons.match_re)

test.write('SConstruct', """\
DefaultEnvironment(tools=[])
def build(env, target, source):
    for t in target:
        with open(str(target[0]), 'wb') as f:
            for s in source:
                with open(str(s), 'rb') as infp:
                    f.write(infp.read())

B = Builder(action=build, multi=1)
env = Environment(tools=[], BUILDERS = { 'B' : B })
env.B(target = ['file10a.out', 'file10b.out'], source = 'file10.in')
env.B(target = ['file10b.out', 'file10a.out'], source = 'file10.in')
""")

test.write('file10.in', 'file10.in\n')

expect = TestSCons.re_escape("""
scons: *** Two different target lists have a target in common: file10b.out  (from ['file10a.out', 'file10b.out'] and from ['file10b.out', 'file10a.out'])
""") + TestSCons.file_expr

test.run(arguments='file10.out', status=2, stderr=expect)

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/CPPDEFINES/basic.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify basic use of CPPPDEFINES with various data types.
"""

import TestSCons

test = TestSCons.TestSCons()

test.write('SConstruct', """\
test_list = [
    'xyz',
    ['x', 'y', 'z'],
    ['x', ['y', 123], 'z', ('int', '$INTEGER')],
    { 'c' : 3, 'b': None, 'a' : 1 },
]
for i in test_list:
    env = Environment(CPPDEFPREFIX='-D', CPPDEFSUFFIX='', INTEGER=0)
    print(env.Clone(CPPDEFINES=i).subst('$_CPPDEFFLAGS'))
for i in test_list:
    env = Environment(CPPDEFPREFIX='|', CPPDEFSUFFIX='|', INTEGER=1)
    print(env.Clone(CPPDEFINES=i).subst('$_CPPDEFFLAGS'))
""")

expect = test.wrap_stdout(build_str="scons: `.' is up to date.\n",
                          read_str = """\
-Dxyz
-Dx -Dy -Dz
-Dx -Dy=123 -Dz -Dint=0
-Da=1 -Db -Dc=3
|xyz|
|x| |y| |z|
|x| |y=123| |z| |int=1|
|a=1| |b| |c=3|
""")

test.run(arguments = '.', stdout=expect)

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/Configure/help.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify that we don't perform Configure context actions when the
-H, -h or --help options have been specified.
"""

import TestSCons

test = TestSCons.TestSCons(match = TestSCons.match_re_dotall)

test.write('SConstruct', """\
env = Environment()
import os
env.AppendENVPath('PATH', os.environ['PATH'])
conf = Configure(env, help=int(ARGUMENTS['help']))
r1 = conf.CheckCHeader( 'math.h' )
r2 = conf.CheckCHeader( 'no_std_c_header.h' ) # leads to compile error
env = conf.Finish()
Export( 'env' )
SConscript( 'SConscript' )
""")

test.write('SConscript', """\
Import( 'env' )
env.Program( 'TestProgram', 'TestProgram.c' )
""")

test.write('TestProgram.c', """\
#include <stdio.h>

int main(void) {
  printf( "Hello\\n" );
}
""")

lines = [
    "Checking for C header file math.h... ",
    "Checking for C header file no_std_c_header.h... "
]

# The help setting should have no effect on -H, so the -H output
# should never contain the lines.
test.run(arguments = '-H help=0')
test.must_not_contain_any_line(test.stdout(), lines)

test.run(arguments = '-H help=1')
test.must_not_contain_any_line(test.stdout(), lines)

# For -h and --help, the lines appear or not depending on how Configure()
# is initialized.
test.run(arguments = '-h help=0')
test.must_not_contain_any_line(test.stdout(), lines)

test.run(arguments = '-h help=1')
test.must_contain_all_lines(test.stdout(), lines)

test.run(arguments = '--help help=0')
test.must_not_contain_any_line(test.stdout(), lines)

test.run(arguments = '--help help=1')
test.must_contain_all_lines(test.stdout(), lines)

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/Clean/Option.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify that {Set,Get}Option('clean') works correctly to control
cleaning behavior.
"""

import os

import TestSCons

_python_ = TestSCons._python_

test = TestSCons.TestSCons()

test.write('build.py', r"""
import sys
with open(sys.argv[1], 'wb') as ofp, open(sys.argv[2], 'rb') as ifp:
    ofp.write(ifp.read())
""")

test.write('SConstruct', """
DefaultEnvironment(tools=[])
B = Builder(action = r'%(_python_)s build.py $TARGETS $SOURCES')
env = Environment(tools=[], BUILDERS = { 'B' : B })
env.B(target = 'foo.out', source = 'foo.in')

mode = ARGUMENTS.get('MODE')
if mode == 'not':
    assert not GetOption('clean')
if mode == 'set-zero':
    assert GetOption('clean')
    SetOption('clean', 0)
    assert GetOption('clean')
if mode == 'set-one':
    assert not GetOption('clean')
    SetOption('clean', 1)
    assert GetOption('clean')
""" % locals())

test.write('foo.in', '"Foo", I say!\n')

test.run(arguments='foo.out MODE=not')
test.must_match(test.workpath('foo.out'), '"Foo", I say!\n')

test.run(arguments='-c foo.out MODE=set-zero')
test.must_not_exist(test.workpath('foo.out'))

test.run(arguments='foo.out MODE=none')
test.must_match(test.workpath('foo.out'), '"Foo", I say!\n')

test.run(arguments='foo.out MODE=set-one')
test.must_not_exist(test.workpath('foo.out'))

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/packaging/rpm/cleanup.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Assert that files created by the RPM packager will be removed by 'scons -c'.
"""

import os
import TestSCons
import SCons.Tool.rpmutils

_python_ = TestSCons._python_
test = TestSCons.TestSCons()

scons = test.program

# TODO: skip this test, since only the intermediate directory needs to be
# removed.

rpm = test.Environment().WhereIs('rpm')

if not rpm:
    test.skip_test('rpm not found, skipping test\n')

rpm_build_root = test.workpath('rpm_build_root')

test.subdir('src')
mainpath = os.path.join('src', 'main.c')
test.file_fixture(mainpath, mainpath)

test.write('SConstruct', """
env=Environment(tools=['default', 'packaging'])

env['ENV']['RPM_BUILD_ROOT'] = r'%(rpm_build_root)s/foo-1.2.3'

env.Prepend(RPM = 'TAR_OPTIONS=--wildcards ')
env.Append(RPMFLAGS = r' --buildroot %(rpm_build_root)s')

prog = env.Install( '/bin/' , Program( 'src/main.c')  )

env.Package( NAME           = 'foo',
             VERSION        = '1.2.3',
             PACKAGEVERSION = 0,
             PACKAGETYPE    = 'rpm',
             LICENSE        = 'gpl',
             SUMMARY        = 'balalalalal',
             X_RPM_GROUP    = 'Application/fu',
             X_RPM_INSTALL  = r'%(_python_)s %(scons)s --install-sandbox="$RPM_BUILD_ROOT" "$RPM_BUILD_ROOT"',
             DESCRIPTION    = 'this should be really really long',
             source         = [ prog ],
             SOURCE_URL     = 'http://foo.org/foo-1.2.3.tar.gz'
            )

env.Alias( 'install', prog )
""" % locals())

# first run: build the package
# second run: make sure everything is up-to-date (sanity check)
# third run: test if the intermediate files have been cleaned
test.run( arguments='.' )
test.up_to_date( arguments='.' )
test.run( arguments='-c .' )

src_rpm     = 'foo-1.2.3-0.src.rpm'
machine_rpm = 'foo-1.2.3-0.%s.rpm' % SCons.Tool.rpmutils.defaultMachine()

test.must_not_exist( machine_rpm )
test.must_not_exist( src_rpm )
test.must_not_exist( 'foo-1.2.3.tar.gz' )
test.must_not_exist( 'foo-1.2.3.spec' )
test.must_not_exist( 'foo-1.2.3/foo-1.2.3.spec' )
test.must_not_exist( 'foo-1.2.3/SConstruct' )
test.must_not_exist( 'foo-1.2.3/src/main.c' )
# We don't remove the directories themselves.  Yet.
#test.must_not_exist( 'foo-1.2.3' )
#test.must_not_exist( 'foo-1.2.3/src' )
test.must_not_exist( 'bin/main' )

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/sconsign/script/no-SConsignFile.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify that the sconsign script works when using an individual
.sconsign file in each directory (SConsignFile(None)).
"""

import TestSCons
import TestSConsign

_python_ = TestSCons._python_

test = TestSConsign.TestSConsign(match = TestSConsign.match_re)

test.subdir('sub1', 'sub2')

# Because this test sets SConsignFile(None), we execute our fake
# scripts directly, not by feeding them to the Python executable.
# That is, we chmod 0o755 and use a "#!/usr/bin/env python" first
# line for POSIX systems, and add .PY to the %PATHEXT% variable on
# Windows.  If we didn't do this, then running this script with
# suitable prileveges would create a .sconsign file in the directory
# where the Python executable lives.  This can happen out of the
# box on Mac OS X, with the result that the .sconsign statefulness
# can mess up other tests.

fake_cc_py = test.workpath('fake_cc.py')
fake_link_py = test.workpath('fake_link.py')

test.write(fake_cc_py, r"""#!%(_python_)s
import os
import re
import sys

path = sys.argv[1].split()

def find_file(f):
    for dir in path:
        p = dir + os.sep + f
        if os.path.exists(p):
            return open(p, 'r')
    return None

def process(infp, outfp):
    for line in infp.readlines():
        m = re.match(r'#include <(.*)>', line)
        if m:
            file = m.group(1)
            found = find_file(file)
            process(found, outfp)
            if found:
                found.close()
        else:
            outfp.write(line)

with open(sys.argv[2], 'w') as outf, open(sys.argv[3], 'r') as ifp:
    outf.write('fake_cc.py:  %%s\n' %% sys.argv)
    process(ifp, outf)

sys.exit(0)
""" % locals())

test.write(fake_link_py, r"""#!%(_python_)s
import sys

with open(sys.argv[1], 'w') as outf, open(sys.argv[2], 'r') as ifp:
    outf.write('fake_link.py:  %%s\n' %% sys.argv)
    outf.write(ifp.read())

sys.exit(0)
""" % locals())

test.chmod(fake_cc_py, 0o755)
test.chmod(fake_link_py, 0o755)

# Note:  We don't use os.path.join() representations of the file names
# in the expected output because paths in the .sconsign files are
# canonicalized to use / as the separator.

sub1_hello_c    = 'sub1/hello.c'
sub1_hello_obj  = 'sub1/hello.obj'
sub2_hello_c    = 'sub2/hello.c'
sub2_hello_obj  = 'sub2/hello.obj'
sub2_inc1_h     = 'sub2/inc1.h'
sub2_inc2_h     = 'sub2/inc2.h'

test.write(['SConstruct'], """
SConsignFile(None)
env1 = Environment(PROGSUFFIX = '.exe',
                   OBJSUFFIX = '.obj',
                   # Specify the command lines with lists-of-lists so
                   # finding the implicit dependencies works even with
                   # spaces in the fake_*_py path names.
                   CCCOM = [[r'%(fake_cc_py)s', 'sub2', '$TARGET', '$SOURCE']],
                   LINKCOM = [[r'%(fake_link_py)s', '$TARGET', '$SOURCE']])
env1.PrependENVPath('PATHEXT', '.PY')
env1.Program('sub1/hello.c')
env2 = env1.Clone(CPPPATH = ['sub2'])
env2.Program('sub2/hello.c')
""" % locals())

test.write(['sub1', 'hello.c'], r"""\
sub1/hello.c
""")

test.write(['sub2', 'hello.c'], r"""\
#include <inc1.h>
#include <inc2.h>
sub2/hello.c
""")

test.write(['sub2', 'inc1.h'], r"""\
#define STRING1 "inc1.h"
""")

test.write(['sub2', 'inc2.h'], r"""\
#define STRING2 "inc2.h"
""")

test.run(arguments = '--implicit-cache --tree=prune .')

sig_re = r'[0-9a-fA-F]{32}'

expect = r"""hello.c: %(sig_re)s \d+ \d+
hello.exe: %(sig_re)s \d+ \d+
        %(sub1_hello_obj)s: %(sig_re)s \d+ \d+
        fake_link\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
hello.obj: %(sig_re)s \d+ \d+
        %(sub1_hello_c)s: %(sig_re)s \d+ \d+
        fake_cc\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
""" % locals()

test.run_sconsign(arguments = "sub1/.sconsign", stdout=expect)

test.run_sconsign(arguments = "--raw sub1/.sconsign",
         stdout = r"""hello.c: {'csig': '%(sig_re)s', 'timestamp': \d+L?, 'size': \d+L?, '_version_id': 2}
hello.exe: {'csig': '%(sig_re)s', 'timestamp': \d+L?, 'size': \d+L?, '_version_id': 2}
        %(sub1_hello_obj)s: {'csig': '%(sig_re)s', 'timestamp': \d+L?, 'size': \d+L?, '_version_id': 2}
        fake_link\.py: {'csig': '%(sig_re)s', 'timestamp': \d+L?, 'size': \d+L?, '_version_id': 2}
        %(sig_re)s \[.*\]
hello.obj: {'csig': '%(sig_re)s', 'timestamp': \d+L?, 'size': \d+L?, '_version_id': 2}
        %(sub1_hello_c)s: {'csig': '%(sig_re)s', 'timestamp': \d+L?, 'size': \d+L?, '_version_id': 2}
        fake_cc\.py: {'csig': '%(sig_re)s', 'timestamp': \d+L?, 'size': \d+L?, '_version_id': 2}
        %(sig_re)s \[.*\]
""" % locals())

test.run_sconsign(arguments = "-v sub1/.sconsign",
         stdout = r"""hello.c:
    csig: %(sig_re)s
    timestamp: \d+
    size: \d+
hello.exe:
    csig: %(sig_re)s
    timestamp: \d+
    size: \d+
    implicit:
        %(sub1_hello_obj)s:
            csig: %(sig_re)s
            timestamp: \d+
            size: \d+
        fake_link\.py:
            csig: %(sig_re)s
            timestamp: \d+
            size: \d+
    action: %(sig_re)s \[.*\]
hello.obj:
    csig: %(sig_re)s
    timestamp: \d+
    size: \d+
    implicit:
        %(sub1_hello_c)s:
            csig: %(sig_re)s
            timestamp: \d+
            size: \d+
        fake_cc\.py:
            csig: %(sig_re)s
            timestamp: \d+
            size: \d+
    action: %(sig_re)s \[.*\]
""" % locals())

test.run_sconsign(arguments = "-c -v sub1/.sconsign",
         stdout = r"""hello.c:
    csig: %(sig_re)s
hello.exe:
    csig: %(sig_re)s
hello.obj:
    csig: %(sig_re)s
""" % locals())

test.run_sconsign(arguments = "-s -v sub1/.sconsign",
         stdout = r"""hello.c:
    size: \d+
hello.exe:
    size: \d+
hello.obj:
    size: \d+
""" % locals())

test.run_sconsign(arguments = "-t -v sub1/.sconsign",
         stdout = r"""hello.c:
    timestamp: \d+
hello.exe:
    timestamp: \d+
hello.obj:
    timestamp: \d+
""" % locals())

test.run_sconsign(arguments = "-e hello.obj sub1/.sconsign",
         stdout = r"""hello.obj: %(sig_re)s \d+ \d+
        %(sub1_hello_c)s: %(sig_re)s \d+ \d+
        fake_cc\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
""" % locals())

test.run_sconsign(arguments = "-e hello.obj -e hello.exe -e hello.obj sub1/.sconsign",
         stdout = r"""hello.obj: %(sig_re)s \d+ \d+
        %(sub1_hello_c)s: %(sig_re)s \d+ \d+
        fake_cc\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
hello.exe: %(sig_re)s \d+ \d+
        %(sub1_hello_obj)s: %(sig_re)s \d+ \d+
        fake_link\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
hello.obj: %(sig_re)s \d+ \d+
        %(sub1_hello_c)s: %(sig_re)s \d+ \d+
        fake_cc\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
""" % locals())

test.run_sconsign(arguments = "sub2/.sconsign",
         stdout = r"""hello.c: %(sig_re)s \d+ \d+
hello.exe: %(sig_re)s \d+ \d+
        %(sub2_hello_obj)s: %(sig_re)s \d+ \d+
        fake_link\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
hello.obj: %(sig_re)s \d+ \d+
        %(sub2_hello_c)s: %(sig_re)s \d+ \d+
        %(sub2_inc1_h)s: %(sig_re)s \d+ \d+
        %(sub2_inc2_h)s: %(sig_re)s \d+ \d+
        fake_cc\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
inc1.h: %(sig_re)s \d+ \d+
inc2.h: %(sig_re)s \d+ \d+
""" % locals())

#test.run_sconsign(arguments = "-i -v sub2/.sconsign",
#         stdout = r"""hello.c: %(sig_re)s \d+ \d+
#hello.exe: %(sig_re)s \d+ \d+
#    implicit:
#        hello.obj: %(sig_re)s \d+ \d+
#hello.obj: %(sig_re)s \d+ \d+
#    implicit:
#        hello.c: %(sig_re)s \d+ \d+
#        inc1.h: %(sig_re)s \d+ \d+
#        inc2.h: %(sig_re)s \d+ \d+
#""" % locals())

test.run_sconsign(arguments = "-e hello.obj sub2/.sconsign sub1/.sconsign",
         stdout = r"""hello.obj: %(sig_re)s \d+ \d+
        %(sub2_hello_c)s: %(sig_re)s \d+ \d+
        %(sub2_inc1_h)s: %(sig_re)s \d+ \d+
        %(sub2_inc2_h)s: %(sig_re)s \d+ \d+
        fake_cc\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
hello.obj: %(sig_re)s \d+ \d+
        %(sub1_hello_c)s: %(sig_re)s \d+ \d+
        fake_cc\.py: %(sig_re)s \d+ \d+
        %(sig_re)s \[.*\]
""" % locals())

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python

# Query gihub issue tracker for the issues of interest.
# The typical triage query is found on
# https://github.com/scons/scons/wiki/BugParty
# FIXME: this needs reworking for github, and wiki needs updating

# Download the issues from Issuezilla as XML; this creates a file
# named 'issues.xml'.  Run this script in the dir containing
# issues.xml (or pass full path as arg to this script) to translate
# 'issues.xml' into a CSV file named 'editlist.csv'.  Upload the CSV
# into a Google spreadsheet.

# In the spreadsheet:
# Select all the columns and pick "align-->top"
# Select the ID and votes columns and pick "align-->right"
# Select the priority column and pick "align-->center"
# Select the first row and click on the "bold" button
# Grab the lines between the column headers to adjust the column widths
# Grab the sort bar on the far left (just above the "1" for row one)
# and move it down one row.  (Row one becomes a floating header)
# Voila!
from __future__ import print_function

# The team members
# FIXME: These names really should be external to this script
team = sorted('Steven Gary Greg Ken Jim Bill Jason Dirk Anatoly'.split())

# The elements to be picked out of the issue
PickList = [
    # sort key -- these are used to sort the entry
    'target_milestone', 'priority', 'votes_desc', 'creation_ts',
    # payload -- these are displayed
    'issue_id', 'votes', 'issue_type', 'target_milestone',
    'priority', 'assigned_to', 'short_desc',
    ]

# Conbert a leaf element into its value as a text string
# We assume it's "short enough" that there's only one substring
def Value(element):
    v = element.firstChild
    if v is None: return ''
    return v.nodeValue

# Parse the XML issues file and produce a DOM for it
import sys
if len(sys.argv) > 1: xml = sys.argv[1]
else: xml = 'issues.xml'
from xml.dom.minidom import parse
xml = parse(xml)

# Go through the issues in the DOM, pick out the elements we want,
# and put them in our list of issues.
issues = []
for issuezilla in xml.childNodes:
    # The Issuezilla element contains the issues
    if issuezilla.nodeType != issuezilla.ELEMENT_NODE: continue
    for issue in issuezilla.childNodes:
        # The issue elements contain the info for an issue
        if issue.nodeType != issue.ELEMENT_NODE: continue
        # Accumulate the pieces we want to include
        d = {}
        for element in issue.childNodes:
            if element.nodeName in PickList:
                d[element.nodeName] = Value(element)
        # convert 'votes' to numeric, ascending and descending
        try:
            v = int('0' + d['votes'])
        except KeyError:
            pass
        else:
            d['votes_desc'] = -v
            d['votes'] = v
        # Marshal the elements and add them to the list
        issues.append([ d[ix] for ix in PickList ])
issues.sort()

# Transcribe the issues into comma-separated values.
# FIXME: parameterize the output file name
import csv
writer = csv.writer(open('editlist.csv', 'w'))
# header
writer.writerow(['ID', 'Votes', 'Type/Member', 'Milestone',
        'Pri', 'Owner', 'Summary/Comments'])
for issue in issues:
    row = issue[4:]        # strip off sort key
    #row[0] = """=hyperlink("http://scons.tigris.org/issues/show_bug.cgi?id=%s","%s")""" % (row[0],row[0])
    if row[3] == '-unspecified-': row[3] = 'triage'
    writer.writerow(['','','','','','',''])
    writer.writerow(row)
    writer.writerow(['','','consensus','','','',''])
    writer.writerow(['','','','','','',''])
    for member in team: writer.writerow(['','',member,'','','',''])

print("Exported %d issues to editlist.csv.  Ready to upload to Google."%len(issues))

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/SWIG/SWIGPATH.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify that use of SWIGPATH finds dependency files in subdirectories.
"""

import TestSCons

test = TestSCons.TestSCons()

swig = test.where_is('swig')
if not swig:
    test.skip_test('Can not find installed "swig", skipping test.\n')

python = test.where_is('python')
if not python:
    test,skip_test('Can not find installed "python", skipping test.\n')

swig = swig.replace('\\','/')
python = python.replace('\\','/')
test.subdir('inc1', 'inc2')

test.write(['inc2', 'dependency.i'], """\
%module dependency
""")

test.write("dependent.i", """\
%module dependent

%include dependency.i
""")

test.write('SConstruct', """
foo = Environment(SWIGFLAGS='-python',
                  SWIG='%(swig)s',
                  SWIGPATH=['inc1', 'inc2'])
swig = foo.Dictionary('SWIG')
bar = foo.Clone(SWIG = [r'%(python)s', 'wrapper.py', swig])
foo.CFile(target = 'dependent', source = ['dependent.i'])
""" % locals())

test.run()

test.up_to_date(arguments = "dependent_wrap.c")

test.write(['inc1', 'dependency.i'], """\
%module dependency

extern char *dependency_1();
""")

test.not_up_to_date(arguments = "dependent_wrap.c")

test.write(['inc2', 'dependency.i'], """\
%module dependency
extern char *dependency_2();
""")

test.up_to_date(arguments = "dependent_wrap.c")

test.unlink(['inc1', 'dependency.i'])

test.not_up_to_date(arguments = "dependent_wrap.c")

test.up_to_date(arguments = "dependent_wrap.c")



test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:


#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/LEX/LEXFLAGS.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import os
import sys

import TestSCons

_python_ = TestSCons._python_
_exe   = TestSCons._exe

test = TestSCons.TestSCons()

test.subdir('in')

test.write('mylex.py', """
import getopt
import sys
import os
if sys.platform == 'win32':
    longopts = ['nounistd']
else:
    longopts = []
cmd_opts, args = getopt.getopt(sys.argv[1:], 'I:tx', longopts)
opt_string = ''
i_arguments = ''
for opt, arg in cmd_opts:
    if opt == '-I': i_arguments = i_arguments + ' ' + arg
    else: opt_string = opt_string + ' ' + opt
for a in args:
    with open(a, 'r') as f:
        contents = f.read()
    contents = contents.replace('LEXFLAGS', opt_string)
    contents = contents.replace('I_ARGS', i_arguments)
    sys.stdout.write(contents)
sys.exit(0)
""")

test.write('SConstruct', """
env = Environment(LEX = r'%(_python_)s mylex.py',
                  LEXFLAGS = '-x -I${TARGET.dir} -I${SOURCE.dir}',
                  tools=['default', 'lex'])
env.CFile(target = 'out/aaa', source = 'in/aaa.l')
""" % locals())

test.write(['in', 'aaa.l'], "aaa.l\nLEXFLAGS\nI_ARGS\n")

test.run('.', stderr = None)

lexflags = ' -x -t'
if sys.platform == 'win32':
    lexflags = ' --nounistd' + lexflags
# Read in with mode='r' because mylex.py implicitley wrote to stdout
# with mode='w'.
test.must_match(['out', 'aaa.c'],	"aaa.l\n%s\n out in\n" % lexflags, mode='r')



test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/AddOption/optional-arg.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify use of the nargs='?' keyword argument to specify a long
command-line option with an optional argument value.
"""

import TestSCons

test = TestSCons.TestSCons()

test.write('SConstruct', """\
AddOption('--install',
          nargs='?',
          dest='install',
          default='/default/directory',
          const='/called/default/directory',
          action='store',
          type='string',
          metavar='DIR',
          help='installation directory')
print(GetOption('install'))
""")

test.run('-Q -q',
         stdout="/default/directory\n")

test.run('-Q -q next-arg',
         stdout="/default/directory\n",
         status=1)

test.run('-Q -q . --install',
         stdout="/called/default/directory\n")

test.run('-Q -q . --install next-arg',
         stdout="/called/default/directory\n",
         status=1)

test.run('-Q -q . first-arg --install',
         stdout="/called/default/directory\n",
         status=1)

test.run('-Q -q . first-arg --install next-arg',
         stdout="/called/default/directory\n",
         status=1)

test.run('-Q -q . --install=/command/line/directory',
         stdout="/command/line/directory\n")

test.run('-Q -q . --install=/command/line/directory next-arg',
         stdout="/command/line/directory\n",
         status=1)

test.run('-Q -q . first-arg --install=/command/line/directory',
         stdout="/command/line/directory\n",
         status=1)

test.run('-Q -q . first-arg --install=/command/line/directory next-arg',
         stdout="/command/line/directory\n",
         status=1)


test.write('SConstruct', """\
AddOption('-X', nargs='?')
""")

expect = r"""
scons: \*\*\* option -X: nargs='\?' is incompatible with short options
File "[^"]+", line \d+, in \S+
"""

test.run(status=2, stderr=expect, match=TestSCons.match_re)



test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Searches through the whole doc/user tree and creates
# all output files for the single examples.
#
from __future__ import print_function

import os
import sys
import SConsExamples

if __name__ == "__main__":
    print("Checking whether all example names are unique...")
    if SConsExamples.exampleNamesAreUnique(os.path.join('doc','user')):
        print("OK")
    else:
        print("Not all example names and suffixes are unique! Please correct the errors listed above and try again.")
        sys.exit(1)

    SConsExamples.createAllExampleOutputs(os.path.join('doc','user'))
